local DUF_GameVersion
local _, _, _, tocversion = GetBuildInfo()
if (tocversion < 20000) then
	DUF_GameVersion = "CLASSIC"
end
if (tocversion > 19999) and (tocversion < 30000) then
	DUF_GameVersion = "TBC"
end
if (tocversion > 29999) and (tocversion < 40000) then
	DUF_GameVersion = "WOTLK"
end
if (tocversion > 39999) then
	DUF_GameVersion = "RETAIL"
end

local DUFsizeX, DUFsizeY = 162, 20
local hasUI
local PetXP
local PetAlpha
local DUF_UNIT_HAPPINESS
local DUF_UNIT_HEALTH_FREQUENT

if (DUF_GameVersion == "CLASSIC") or (DUF_GameVersion == "TBC") then
	DUF_UNIT_HEALTH_FREQUENT = "UNIT_HEALTH_FREQUENT"
	DUF_UNIT_HAPPINESS = "UNIT_HAPPINESS"
else
	DUF_UNIT_HEALTH_FREQUENT = "UNIT_HEALTH"
	DUF_UNIT_HAPPINESS = "UNIT_POWER_UPDATE"
end

local function DUF_Mouseover(self, unit) -- Show/Hide on mouseover when NOT in combat
	if self:IsMouseOver() then
		self:SetAlpha(1)
	end
	if (unit == "player") or (unit == "pet") then
		if ( UnitAffectingCombat(unit) ) then
			self:EnableMouse(false)
			self:SetAlpha(1)
		else
			self:EnableMouse(true)
			if _G["FizzleHealthButton"]:IsMouseOver() then _G["FizzleHealth"]:SetAlpha(1) end
			if _G["FizzlePetHealthButton"]:IsMouseOver() then _G["FizzlePetHealth"]:SetAlpha(1) end
		end
	end
end

--Base code derived from Fizzlemizz's official forums post :)
local FocusHealth
local FocusPower

local function CreateBar(name, sizeX, sizeY, statusfontSize, namefontSize, markersize, previous) -- Create StatusBar with a text overlay
	local f = CreateFrame("StatusBar", "Fizzle"..name, UIParent)
		f:SetSize(sizeX, sizeY)
		--statusbar background
		f.bg = f:CreateTexture(nil, "BACKGROUND")
		f.bg:SetAllPoints(f)
		f.bg:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
		f.bg:SetColorTexture(0, 0, 0, 0.5)
		
		--statusbar texture
		f:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
		
		-- statusbar text
		-- left
		f.TextL = f:CreateFontString("FontString","OVERLAY","GameTooltipText")
		f.TextL:SetFont("Fonts\\FRIZQT__.TTF", statusfontSize, "THINOUTLINE")
		f.TextL:SetTextColor(1, 1, 1);
		f.TextL:SetPoint("LEFT", f, "LEFT", 0, 0)		
		f.TextL:SetJustifyH("LEFT")
		f.TextL:SetJustifyV("CENTER")
		
		-- center
		f.TextC = f:CreateFontString("FontString","OVERLAY","GameTooltipText")
		f.TextC:SetFont("Fonts\\FRIZQT__.TTF", statusfontSize, "THINOUTLINE")
		f.TextC:SetTextColor(1, 1, 1);
		f.TextC:SetPoint("CENTER", f, "CENTER", 0, 0)		
		f.TextC:SetJustifyH("CENTER")
		f.TextC:SetJustifyV("CENTER")
		
		-- right
		f.TextR = f:CreateFontString("FontString","OVERLAY","GameTooltipText")
		f.TextR:SetFont("Fonts\\FRIZQT__.TTF", statusfontSize, "THINOUTLINE")
		f.TextR:SetTextColor(1, 1, 1);
		f.TextR:SetPoint("RIGHT", f, "RIGHT", 0, 0)		
		f.TextR:SetJustifyH("RIGHT")
		f.TextR:SetJustifyV("CENTER")
		
		-- unit name
		f.unitname = f:CreateFontString("FontString","OVERLAY","GameTooltipText")
		f.unitname:SetFont("Fonts\\FRIZQT__.TTF", namefontSize, "THINOUTLINE")
		f.unitname:SetPoint("LEFT", f, "LEFT", 0, namefontSize)	
		
		-- target mark
		f.targetmark = f:CreateTexture(nil, "BACKGROUND")
		f.targetmark:SetSize(markersize, markersize)

		-- leader icon
		f.leadassisticon = f:CreateTexture(nil, "BACKGROUND",nil,-8)
		f.leadassisticon:SetSize(markersize*0.7, markersize*0.7)

		-- looter icon
		f.lootericon = f:CreateTexture(nil, "BACKGROUND",nil,-7)
		f.lootericon:SetSize(markersize*0.5, markersize*0.5)

		-- maintank-assist icon
		f.mtaicon = f:CreateTexture(nil, "BACKGROUND",nil,-8)
		f.mtaicon:SetSize(markersize*0.5, markersize*0.5)

		-- pvp icon
		f.pvpicon = f:CreateTexture(nil, "BACKGROUND")
		f.pvpicon:SetSize(markersize*2, markersize*2)

		-- target rare dragon
		f.targetrare = f:CreateTexture(nil, "BACKGROUND")
		f.targetrare:SetDrawLayer("BACKGROUND", -8)
		f.targetrare:SetSize(DUFsizeY*3, DUFsizeY*3)

		-- unit level
		f.unitlevel = f:CreateFontString("FontString","OVERLAY","GameTooltipText")
		f.unitlevel:SetFont("Fonts\\FRIZQT__.TTF", statusfontSize, "THINOUTLINE")
		f.unitlevel:SetJustifyV("CENTER")

		-- pet happiness
		-- Use PetAlpha so we can show and hide it independent of the bar it is attached to.
		f.pethappiness = f:CreateFontString("FontString","OVERLAY","GameTooltipText")
		f.pethappiness:SetJustifyV("RIGHT")
	
	if (name == "Health") or (name == "PetHealth") or 
		(name == "THealth") or (name == "ToTHealth") or
		(name == "FocusHealth") then
		-- Secure Button overlay
		--Create the unit button:
		local fsbo = CreateFrame("Button", "Fizzle"..name.."Button", f, "SecureUnitButtonTemplate")
		-- Tell it which unit to represent (in this case "player" and "pet:
		if (name == "Health") then
			f:SetPoint("TOPRIGHT", UIParent, "CENTER", -198, 0)
			fsbo:SetAttribute("unit", "player")
			fsbo:SetScript('OnEnter', function() f:SetAlpha(1) end)
			fsbo:SetScript('OnLeave', function() 
				if ( UnitAffectingCombat("player") ) then
					f:SetAlpha(1)
				else
					f:SetAlpha(0.25)
				end
			end)
		elseif (name == "FocusHealth") then
			f:SetPoint("TOPRIGHT", "FizzleHealth", "TOPLEFT", ( -(DUFsizeX*0.415) ), 0 ) 
			fsbo:SetAttribute("unit", "focus")
			f:SetAlpha(1)
			f:Hide()
		elseif (name == "PetHealth") then
			f:SetPoint("TOPLEFT", "FizzleHealth", "BOTTOMLEFT", 0, -( (DUFsizeY*0.7) + 66 ) )
			fsbo:SetAttribute("unit", "pet")
			fsbo:SetScript('OnEnter', function() f:SetAlpha(1) end)
			fsbo:SetScript('OnLeave', function() 
				if ( UnitAffectingCombat("pet") ) then
					f:SetAlpha(1)
				else
					f:SetAlpha(0.25)
				end
			end)
		elseif (name == "THealth") then
			f:SetPoint("TOPLEFT", UIParent, "CENTER", 198, 0)
			fsbo:SetAttribute("unit", "target")
		elseif (name == "ToTHealth") then
			f:SetPoint("TOPRIGHT", "FizzleTHealth", "BOTTOMRIGHT", 0, -( (DUFsizeY*0.7) + 66 ) )
			fsbo:SetAttribute("unit", "targettarget")
		end
		
		fsbo:SetAllPoints(f)		
		-- Tell the game to "look after it"
		RegisterUnitWatch(fsbo) 		
		-- Tell it to show the unit's vehicle when the unit is in one:
		fsbo:SetAttribute("toggleForVehicle", true)		
		-- Give it the standard click actions:
		fsbo:RegisterForClicks("AnyUp")
		fsbo:SetAttribute("*type1", "target") -- Target unit on left click
		fsbo:SetAttribute("*type2", "togglemenu") -- Toggle units menu on right click
		fsbo:SetAttribute("*type3", "assist") -- On middle click, target the target of the clicked unit
	elseif (name == "PetAlpha") then
		f:SetSize(sizeX*0.3, sizeY)
		f.bg:SetTexture(nil)
		f:SetPoint("TOPRIGHT", "FizzlePetHealth", "BOTTOMLEFT", 0, 0)
	else
		f:SetPoint("TOP", previous, "BOTTOM")
	end
	if ( (name == "Power") or (name == "DruidMana") or 
		(name == "PetPower") or (name == "PetXP") or (name == "PetAlpha") ) then
			f:SetScript('OnEnter', function() f:SetAlpha(1) end)
			f:SetScript('OnLeave', function() 
				if ( UnitAffectingCombat("player") ) then
					f:SetAlpha(1)
				else
					f:SetAlpha(0.25)
				end
			end)
	elseif (name == "FocusPower") then
		f:Hide()
		f:SetAlpha(1)
	end
	return f
end

--########## Health #############
local function UpdateHealth(self, unit) -- Update the health bar
	self:Show()
	self:SetStatusBarColor(0, 1, 0) -- make health status bars green
	self.leadassisticon:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, -2)
	self.lootericon:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 20, 1)
	self.mtaicon:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 36, 1)
	self.pvpicon:SetPoint("CENTER", self, "TOPLEFT", -(DUFsizeX/8), -(DUFsizeY/2))
	local name, realm = GetUnitName(unit, true)
	local function checkName() -- Classic Warlock pet names return "Unknown" on login.
		name, realm = GetUnitName(unit, true)
	end
	if (name == "Unknown") then -- Classic Warlock pet names return "Unknown" on login.
		C_Timer.After(1, checkName)
	elseif (unit == "player") then
		self.unitname:SetText("")
	elseif (unit == "focus") then
		if string.len(name) > 11 then
			self.unitname:SetText(string.sub(name, 1, 11) .. "...") -- Limits name length to 12 characters
		else
			self.unitname:SetText(name)
		end
	else
		self.unitname:SetText(name)
	end
	local className, classFilename, classId = UnitClass(unit)
	local rPerc, gPerc, bPerc, argbHex = GetClassColor(classFilename)
	self.unitname:SetTextColor(rPerc, gPerc, bPerc);
	local health = UnitHealth(unit)
	self.healthMax = UnitHealthMax(unit)
	self:SetMinMaxValues(0, self.healthMax)
	self:SetValue(health)
	self.TextL:SetFormattedText("%.0f%%", ceil( (health/self.healthMax) * 100 )  ) -- ceil math function to match Blizzard %s
	self.TextR:SetText(health)
	local level = UnitLevel(unit)
	self.targetrare:Hide()
	if (level == -1) then
		level = "|TInterface\\TargetingFrame\\UI-TargetingFrame-Skull:16|t" -- Bosses or units +10 levels above player.
	end
	local maxlevel = GetMaxPlayerLevel()	
	self.unitlevel:SetTextColor(rPerc, gPerc, bPerc)
	if (unit == "player") or (unit == "pet") then
		self.unitlevel:ClearAllPoints()
		self.unitlevel:SetPoint("CENTER", self, "LEFT", -(DUFsizeX)*0.115, 0)
		self.unitlevel:SetJustifyH("CENTER")
		if (level == maxlevel) then
			self.unitlevel:SetText("")
		else
			self.unitlevel:SetText(level)
		end
		if ( UnitAffectingCombat(unit) ) then
			self:SetAlpha(1)
		else
			if (health == self.healthMax) then
				self:SetAlpha(0.25)
			else
				self:SetAlpha(1)
			end
		end
	end
	-- Raid Target Markers
	local index = GetRaidTargetIndex(unit);
	if (index == nil) then
		if self.targetmark then
			self.targetmark:Hide()
		end
	else
		self.targetmark:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
		SetRaidTargetIconTexture(self.targetmark, index)
		self.targetmark:ClearAllPoints()
		self.targetmark:SetPoint("BOTTOM", self, "LEFT", -(DUFsizeX)*0.12, (DUFsizeY))
		self.targetmark:Show()
	end

	local isLeader = UnitIsGroupLeader(unit)
	local isGroupAssist = UnitIsGroupAssistant(unit)
	if isLeader then
		self.leadassisticon:SetTexture("Interface\\GROUPFRAME\\UI-Group-LeaderIcon")
		self.leadassisticon:Show()
	elseif isGroupAssist then
		self.leadassisticon:SetTexture("Interface\\GROUPFRAME\\UI-GROUP-ASSISTANTICON")
	else
		self.leadassisticon:Hide()
	end

	local isLooter = UnitIsGroupLeader(unit)
	if isLooter then
		self.lootericon:SetTexture("Interface\\GROUPFRAME\\UI-Group-MasterLooter")
		self.lootericon:Show()
	else
		self.lootericon:Hide()
	end
	
	local maintank1, maintank2 = GetPartyAssignment("MAINTANK" ,unit, 1)
	local mainassist1, mainassist2 = GetPartyAssignment("MAINASSIST" ,unit, 1)
	if maintank1 or maintank2 then
		self.mtaicon:SetTexture("Interface\\GROUPFRAME\\UI-GROUP-MAINTANKICON")
		self.mtaicon:Show()
	elseif mainassist1 or mainassist2 then
		self.mtaicon:SetTexture("Interface\\GROUPFRAME\\UI-GROUP-MAINASSISTICON")
		self.mtaicon:Show()
	else
		self.mtaicon:Hide()
	end
	
	local ispvp = UnitIsPVP(unit);
	if ispvp then
		local englishFaction = UnitFactionGroup(unit)
		if englishFaction == "Alliance" then
			self.pvpicon:SetTexture("Interface\\COMMON\\icon-alliance")
			self.pvpicon:Show()
		elseif englishFaction == "Horde" then
			self.pvpicon:SetTexture("Interface\\COMMON\\icon-horde")
			self.pvpicon:Show()
		else
			self.pvpicon:Hide()
		end
	else
		self.pvpicon:Hide()
	end
	DUF_Mouseover(self, unit)
	if (unit == "target") or (unit == "targettarget") or (unit == "focus") then
		self.unitlevel:SetJustifyH("CENTER")
		self.unitlevel:SetText(level)
		self.leadassisticon:SetSize(DUFsizeY*1.4, DUFsizeY*1.4)
		self.leadassisticon:ClearAllPoints()
		self.leadassisticon:SetPoint("TOPRIGHT", self, "TOPLEFT", -4, 6)
		self.lootericon:ClearAllPoints()
		self.lootericon:SetPoint("TOPRIGHT", self, "TOPLEFT", -11.5, -1.75)
		self.mtaicon:ClearAllPoints()
		self.mtaicon:SetSize(DUFsizeY, DUFsizeY)
		self.mtaicon:SetPoint("TOPRIGHT", self, "TOPLEFT", -8, -14)
		self.pvpicon:ClearAllPoints()
		self.pvpicon:SetPoint("CENTER", self, "RIGHT", (DUFsizeX*0.1), 0)
		-- Target Unit Reactions
		local reaction = UnitReaction("player", unit)
		if (reaction) and not UnitPlayerControlled(unit) then 
			if reaction < 4 then 
				self.unitname:SetTextColor(1, 0, 0)
				self.unitlevel:SetTextColor(1, 0, 0)
			else
				self.unitname:SetTextColor(1, 1, 0)
				self.unitlevel:SetTextColor(1, 1, 0)
			end
		end
		-- Target Unit Boss Indicator
		local raremob = UnitClassification(unit);
		local raretitle = ""
		if (raremob == "worldboss") or (raremob == "rareelite") or (raremob == "elite") or (raremob == "rare") then
			if raremob == "worldboss" then 
				raretitle = "World Boss"
				self.unitlevel:SetTextColor(0.83, 0.68, 0.21, 1)
				self.unitlevel:SetText("|TInterface\\TargetingFrame\\UI-TargetingFrame-Skull:16|t".."("..raretitle..")")
			elseif raremob == "rareelite" then
				raretitle = "Rare Elite"
				self.unitlevel:SetTextColor(0.47, 0.47, 0.47, 1)
				self.unitlevel:SetText(level.." ("..raretitle..")")
				self.targetrare:SetTexture("Interface\\AddOns\\DejaUFClassic\\Art\\DejaUF-UI-TargetingFrame-Rare-Elite")
				self.targetrare:Show()
			elseif raremob == "elite" then
				raretitle = "Elite"
				self.unitlevel:SetTextColor(0.82, 0.68, 0.21, 1)
				self.unitlevel:SetText(level.."\n"..raretitle)
				self.targetrare:SetTexture("Interface\\AddOns\\DejaUFClassic\\Art\\DejaUF-UI-TargetingFrame-Elite")
				self.targetrare:Show()
			elseif raremob == "rare" then
				raretitle = "Rare"
				self.unitlevel:SetTextColor(0.75,0.75,0.75, 1)
				self.unitlevel:SetText(level.." ("..raretitle..")")
				self.targetrare:SetTexture("Interface\\AddOns\\DejaUFClassic\\Art\\DejaUF-UI-TargetingFrame-Rare")
				self.targetrare:Show()
			else
				self.unitlevel:SetTextColor(1, 1, 1, 1)
				self.unitlevel:SetText(level.." ("..raretitle..")")
			end
			if (unit == "focus") then
				self.unitname:SetFont("Fonts\\FRIZQT__.TTF", (DUFsizeX)/( (DUFsizeY)/2 ), "THINOUTLINE")
				self.unitlevel:SetJustifyH("LEFT")
				self.unitlevel:ClearAllPoints()
				self.unitlevel:SetPoint("RIGHT", self, "LEFT", -(DUFsizeX)*0.043, 0)
				self.targetrare:SetPoint("CENTER", self, "TOPLEFT", -(DUFsizeX/6), -(DUFsizeY/2))
				self.targetrare:SetTexCoord(1, 0, 0, 1)
				if raremob == "worldboss" then
					self.unitlevel:SetJustifyH("CENTER")
					self.unitlevel:ClearAllPoints()
					self.unitlevel:SetPoint("BOTTOM", self, "TOP", 0, (DUFsizeY) )
				end
			else
				self.unitlevel:SetJustifyH("RIGHT")
				self.unitlevel:ClearAllPoints()
				-- self.unitlevel:SetPoint("LEFT", self, "RIGHT", (DUFsizeY)*.43, 0)
				self.unitlevel:SetPoint("LEFT", self, "RIGHT", (DUFsizeX)*0.03, 0)
				self.targetrare:SetPoint("CENTER", self, "RIGHT", (DUFsizeX/5), -(DUFsizeY/5))
			end
		else
			self.unitlevel:ClearAllPoints()
			self.unitlevel:SetPoint("CENTER", self, "RIGHT", (DUFsizeX)*0.11 , 0)
			self.unitlevel:SetText(level)
			self.targetrare:Hide()
			if (unit == "focus") then
				self.unitlevel:ClearAllPoints()
				self.unitlevel:SetPoint("CENTER", self, "LEFT", -(DUFsizeX)*0.115, 0)
			end
		end
		if (unit == "targettarget") then
			self.leadassisticon:Hide()
			self.lootericon:Hide()
			self.mtaicon:Hide()
		end
		if (unit == "focus") then
			self.leadassisticon:Hide()
			self.lootericon:Hide()
			self.mtaicon:Hide()
			self.pvpicon:SetPoint("CENTER", self, "TOPLEFT", -(DUFsizeX/8), -(DUFsizeY/2))
		end
	end
end

--########## Power #############
local function UpdatePower(self, unit) -- Update the power bar
	self:Show()
	local _, _, classIndex = UnitClass(unit)
	local power = UnitPower(unit)
	self.TextC:SetText("")
	self.powerMax = UnitPowerMax(unit, UnitPowerType(unit))
	if (power == self.powerMax) then
		self:SetAlpha(0.25)
	elseif (power > 0) then
		self:SetAlpha(1)
	end
	if (unit == "target") or (unit == "targettarget") or (unit == "focus") then --Interpreted linear top-down fallthru logic
		self:SetAlpha(1)
	end
	--Warrior, Rogue, and Targeted NPC power Frames display
	if (classIndex==1) or (classIndex == 4) then
		if (classIndex==1) then --Warrior
			if (power == 0) then
				self:SetAlpha(0)
			end
			self:SetStatusBarColor(1, 0, 0) -- make it red for rage
			self:SetMinMaxValues(0, (self.powerMax)/10)
		elseif (classIndex == 4) then --Rogue
			self:SetStatusBarColor(1, 1, 0) -- make it yellow for energy
			self:SetMinMaxValues(0, self.powerMax)
		end
		if (unit == "pet") or ( UnitIsUnit("target", "pet") ) or 
		( UnitIsUnit("targettarget", "pet") ) or ( UnitIsUnit("focus", "pet") ) then
			self:SetMinMaxValues(0, (self.powerMax))
		end
		self:SetValue(power)
		self.TextL:SetFormattedText("")
		self.TextC:SetText(power)
		self.TextR:SetText("")
	else
		self:SetStatusBarColor(0, 0, 1) -- make it blue for mana.
		self:SetMinMaxValues(0, self.powerMax)
		self:SetValue(power)
		self.TextL:SetFormattedText("%.0f%%", ( ceil( (power/self.powerMax) * 100 ) ) ) -- ceil math function to match Blizzard %s
		self.TextR:SetText(power)
	end
	if ( UnitIsUnit("targettarget", "pet") ) then
		self:SetMinMaxValues(0, (self.powerMax))
	end
	if (unit == "player") then
		if (classIndex == 11) then
			local Bipedalpower = UnitPower(unit, 0)
			self.BipedalpowerMax = UnitPowerMax(unit, 0) --Getting Bipedal mana
			if (Bipedalpower == self.BipedalpowerMax) then
				self:SetAlpha(0.25)
			else
				self:SetAlpha(1)
			end
			self:SetMinMaxValues(0, self.BipedalpowerMax)
			self.TextL:SetFormattedText("%.0f%%", ( ceil( (Bipedalpower/self.BipedalpowerMax) * 100 ) ) ) -- ceil math function to match Blizzard %s
			self.TextR:SetText(Bipedalpower)
			self:SetValue(Bipedalpower)
			if (DUF_GameVersion == "CLASSIC") or (DUF_GameVersion == "TBC") then
				---------- Number of shapeshifts for current form left
				local shiftcost
				local usable, nomana = IsUsableSpell(24858);
				if (usable) then
					if (nomana) then
						shiftcost = (GetSpellPowerCost(24858)[1].cost)
					else
						shiftcost = (GetSpellPowerCost(768)[1].cost)
					end
				else
					shiftcost = (GetSpellPowerCost(768)[1].cost)
				end
				self.TextC:SetText(floor(Bipedalpower/shiftcost))
			else
				self.TextC:SetText("")
			end
		end
		if UnitAffectingCombat(unit) then
			self:SetAlpha(1)
		end
	end
	DUF_Mouseover(self, unit)
end

--########## DruidMana #############
local function UpdateDruidMana(self, unit) -- Update the DruidMana bar
	if (unit == "player") then
		local druidmana = UnitPower(unit)
		local index = GetShapeshiftFormID()
		self.druidmanaMax = UnitPowerMax(unit, UnitPowerType(unit))
		self.TextC:SetText(druidmana)
		if (index==1) or (index == 8) then
			self:SetAlpha(0.25)
			if (index==1) then --Cat
				if (druidmana < self.druidmanaMax) then
					self:SetAlpha(1)
				end
				self:SetStatusBarColor(1, 1, 0) -- make it yellow for energy
				self:SetMinMaxValues(0, self.druidmanaMax)
			elseif (index == 8) then --Bear
				if druidmana > 0 then
					self:SetAlpha(1)
				elseif druidmana == 0 then
					self:SetAlpha(0)
				end
				self:SetStatusBarColor(1, 0, 0) -- make it red for rage
				self:SetMinMaxValues(0, (self.druidmanaMax)/10)
			end				
			if UnitAffectingCombat(unit) then
				self:SetAlpha(1)
			end
			self:SetValue(druidmana)
			self:Show()
		else
			self:Hide()
		end
		DUF_Mouseover(self, unit)
	end
end

-- PlayerFrame:SetScript("OnShow", function(self) -- Hide PlayerFrame after creation of DejaUFClassic player frame
-- 	self:Hide()
-- end)

-- TargetFrame:SetScript("OnShow", function(self) -- Hide TargetFrame after creation of DejaUFClassic TargetFrame
-- 	self:Hide()
-- end)

-- TargetFrameToT:SetScript("OnShow", function(self) -- Hide TargetFrameToT after creation of DejaUFClassic TargetFrameToT
-- 	self:Hide()
-- end)

--########## Pet XP #############
local function UpdatePetXP(self, unit) -- Update the health bar
	if (unit == "pet") then
		local level = UnitLevel(unit)
		local maxlevel = GetMaxPlayerLevel()	
		if (level < maxlevel) then
			local currXP, nextXP = GetPetExperience()
			self:SetStatusBarColor(0, 0.75, 1) -- make it "Deja Blue" for pet xp
			self:SetMinMaxValues(0, nextXP)
			self:SetValue(currXP)
			self.TextC:SetText( "("..format("%.2f%%", (currXP/nextXP) * 100 )..") "..currXP.."/"..nextXP ) -- ceil math function to match Blizzard %s
			if UnitAffectingCombat(unit) then
				self:SetAlpha(0)
			else
				self:SetAlpha(0.25)
			end
		else
			self:Hide()
		end
		DUF_Mouseover(self, unit)
	else
		self:Hide()
	end
end

--########## Pet Happiness, & Loyalty #############
local function UpdatePetAlpha(self, unit) -- Update the health bar
	if (unit == "pet") then
		local happiness, damagePercentage, loyaltyRate = GetPetHappiness()
		local function SetAlpha()
			if happiness < 3 then
				self:SetAlpha(1)
			else
				self:SetAlpha(0.25)
			end
		end
		if happiness then
			SetAlpha()
			if ( UnitAffectingCombat(unit) ) then
				self:EnableMouse(false)
			else
				self:EnableMouse(true) -- Enable mouseover here since I don't want to allow it in combat
			end
			self:SetScript('OnEnter', function() self:SetAlpha(1) end)
			self:SetScript('OnLeave', function() self:SetAlpha(0.25) SetAlpha() end)

			self:SetStatusBarColor(0, 0, 0, 0) -- make it "transparent for pet happiness
			self:SetMinMaxValues(0, 3)
			self:SetValue(happiness)

			self.pethappiness:SetPoint("CENTER", self, "CENTER", 0, 0)
			self.pethappiness:SetJustifyH("CENTER")
			self.pethappiness:SetFormattedText("%.0f%%", (damagePercentage))
			if happiness == 3 then
				self.pethappiness:SetTextColor(0, 1, 0) -- Green @ 125%
				self.pethappiness:SetFont("Fonts\\FRIZQT__.TTF", 12, "THINOUTLINE")
			elseif happiness == 2 then
				self.pethappiness:SetTextColor(1, 1, 0) -- Yellow @ 100%
				self.pethappiness:SetFont("Fonts\\FRIZQT__.TTF", 16, "THINOUTLINE")
			elseif happiness == 1 then
				self.pethappiness:SetTextColor(1, 0, 0) -- Red @ 75%
				self.pethappiness:SetFont("Fonts\\FRIZQT__.TTF", 18, "THINOUTLINE")
			elseif happiness < 1 then
				self.pethappiness:SetTextColor(0, 0, 0) -- Black @ below 75%
				self.pethappiness:SetFont("Fonts\\FRIZQT__.TTF", 20, "THINOUTLINE")
			end
		end
		DUF_Mouseover(self, unit)
	else
		self:Hide()
	end
end

PetFrame:SetScript("OnShow", function(self) -- Hide PetFrame after creation of DejaUFClassic PetFrame
	self:Hide()
end)

-- Player Frames
-- name, sizeX, sizeY, statusfontSize, namefontSize, markersize, previous
local Health = CreateBar('Health', DUFsizeX, DUFsizeY, DUFsizeY, DUFsizeY*1.25, DUFsizeY*1.5) -- Create the health bar
	Health.PowerPoints = Health:CreateFontString("FontString","OVERLAY","GameTooltipText")
	Health.PowerPoints:SetFont("Fonts\\FRIZQT__.TTF", 27, "THINOUTLINE")
	Health.PowerPoints:SetPoint("LEFT", Health, "RIGHT", 10, 0)		
	Health.PowerPoints:Show()

local Power = CreateBar('Power', DUFsizeX, DUFsizeY*0.7, DUFsizeY*0.7, DUFsizeY*0.8, DUFsizeY*1.25, Health) -- Create the power bar (set to anchor below Health)
local DruidMana = CreateBar('Power', DUFsizeX, DUFsizeY, DUFsizeY, DUFsizeY*1.25, DUFsizeY*1.5, Power) -- Create the power bar (set to anchor below Health)

-- Pet Frames
-- name, sizeX, sizeY, statusfontSize, namefontSize, markersize, previous
local PetHealth = CreateBar('PetHealth', DUFsizeX*0.80, DUFsizeY, DUFsizeY*0.7, DUFsizeY, DUFsizeY*1.25) -- Create the PetHealth bar
local PetPower = CreateBar('PetPower', DUFsizeX*0.80, DUFsizeY*0.7, DUFsizeY*0.5, DUFsizeY, DUFsizeY*1.25, PetHealth) -- Create the PetPower bar (set to anchor below PetHealth)
if (DUF_GameVersion == "CLASSIC") or (DUF_GameVersion == "TBC") then
	PetXP = CreateBar('PetXP', DUFsizeX*0.80, DUFsizeY*0.7, DUFsizeY*0.5, DUFsizeY, DUFsizeY*1.25, PetPower) -- Create the PetPower bar (set to anchor below PetHealth)
	PetAlpha = CreateBar('PetAlpha', DUFsizeX*0.80, DUFsizeY*0.7, DUFsizeY*0.5, DUFsizeY, DUFsizeY*1.25, PetXP) -- Create the PetPower bar (set to anchor below PetHealth)
end

-- Focus Frames
if (not (DUF_GameVersion == "CLASSIC")) then
	-- name, sizeX, sizeY, statusfontSize, namefontSize, markersize, previous
	FocusHealth = CreateBar('FocusHealth', DUFsizeX*0.80, DUFsizeY, DUFsizeY*0.7, DUFsizeY, DUFsizeY*1.25) -- Create the FocusHealth bar
	FocusPower = CreateBar('FocusPower', DUFsizeX*0.80, DUFsizeY*0.7, DUFsizeY*0.5, DUFsizeY, DUFsizeY*1.25, FocusHealth) -- Create the FocusPower bar (set to anchor below FocusHealth)
end

--Target Frames
-- name, sizeX, sizeY, statusfontSize, namefontSize, markersize, previous
local THealth = CreateBar('THealth', DUFsizeX, DUFsizeY, DUFsizeY, DUFsizeY*1.25, DUFsizeY*1.5) -- Create the health bar
	THealth.ComboPoints = THealth:CreateFontString("FontString","OVERLAY","GameTooltipText")
	THealth.ComboPoints:SetFont("Fonts\\FRIZQT__.TTF", 27, "THINOUTLINE")
	THealth.ComboPoints:SetTextColor(1, 0.9609375, 0.4140625);
	THealth.ComboPoints:SetPoint("RIGHT", THealth, "LEFT", -10, 0)		
	THealth.ComboPoints:Show()

local TPower = CreateBar('TPower', DUFsizeX, DUFsizeY*0.7, DUFsizeY*0.7, DUFsizeY*0.8, DUFsizeY*1.25, THealth) -- Create the power bar (set to anchor below Health)

local CombofinishFired = 0
local HealthfinishFired = 0
local _, _, classIndex = UnitClass("player")

local function DUFGetCombopoints(event) -- Classic Combo Points
	if UnitExists("target") and not (UnitIsDeadOrGhost("target")) then
		local health = UnitHealth("target");
		local max_health = UnitHealthMax("target");
		local healthPercent = ( ( health / max_health ) * 100 )
		local comboPoints
		if (healthPercent > 25) then
			HealthfinishFired = 0
		end
		if ( (healthPercent <= 25) and (HealthfinishFired == 0) ) then
			print(healthPercent, HealthfinishFired)
			PlaySoundFile("Interface\\AddOns\\DejaUFClassic\\Sounds\\finishim.wav");
			HealthfinishFired = 1
		end
		if (event == "UNIT_POWER_UPDATE") or (event == "PLAYER_TARGET_CHANGED") then
			if (classIndex == 4) or (classIndex == 11) then
				comboPoints = GetComboPoints("player", "target")
				if (comboPoints > 0) then
					THealth.ComboPoints:SetText(comboPoints)
				else
					THealth.ComboPoints:SetText("")
				end
				if (comboPoints < 4) then
					CombofinishFired = 0
				end
				if ( ( (comboPoints == 4) or (comboPoints == 5) ) and (CombofinishFired == 0) ) then
					PlaySoundFile("Interface\\AddOns\\DejaUFClassic\\Sounds\\finishim.wav");
					CombofinishFired = 1
				end
			end
		end
	end
end

local function DUFGetPowerpoints() --Retail many classes also have combo points; Rogue, Paladin, Warlock, Mage, Monk, etc.
	local Power_Points = {
		4, 5, 7, 9, 12, 16,
	}
	for _, v in ipairs(Power_Points) do
		local powerpoints = UnitPower("player", v)
		if (powerpoints > 0) then
			local _, classFilename = UnitClass("player")
			local rPerc, gPerc, bPerc, argbHex = GetClassColor(classFilename)
			Health.PowerPoints:SetTextColor(rPerc, gPerc, bPerc);
			Health.PowerPoints:SetText(powerpoints)
			return
		else
			Health.PowerPoints:SetText("")
		end
	end
end

local hasUI, isHunterPet
local function UpdatePetUnit(hasUI, isHunterPet, unit, event)
	if hasUI then
		if unit == "pet" then
			if not ( UnitAffectingCombat("pet") ) then
				PetHealth:Show()
				PetPower:Show()
			end
			UpdateHealth(PetHealth, "pet")
			UpdatePower(PetPower, "pet")
			if (DUF_GameVersion == "CLASSIC") or (DUF_GameVersion == "TBC") then
				if isHunterPet then
					-- Still may need a Pet XP event workaround.
					if not ( UnitAffectingCombat("pet") ) then
						PetXP:Show()
						PetAlpha:Show()
					end
					UpdatePetXP(PetXP, "pet")
					UpdatePetAlpha(PetAlpha, "pet")
				end
			end
		end
	else
		PetHealth:Hide()
		PetPower:Hide()
		if (DUF_GameVersion == "CLASSIC") or (DUF_GameVersion == "TBC") then
			PetXP:Hide()
			PetAlpha:Hide()
		end
	end
end

--Target of target Frames
-- name, sizeX, sizeY, statusfontSize, namefontSize, markersize, previous
local ToTHealth = CreateBar('ToTHealth', DUFsizeX*0.80, DUFsizeY, DUFsizeY*0.7, DUFsizeY, DUFsizeY*1.25) -- Create the health bar
local ToTPower = CreateBar('ToTPower', DUFsizeX*0.80, DUFsizeY*0.7, DUFsizeY*0.5, DUFsizeY, DUFsizeY*1.25, ToTHealth) -- Create the power bar (set to anchor below Health)

------------------------
-- ##### EVENTS ##### --
------------------------

	Health:RegisterUnitEvent("PLAYER_LOGIN") -- register the events to be used
	Health:RegisterUnitEvent("PLAYER_ENTERING_WORLD") -- register the events to be used
	Health:RegisterUnitEvent("PLAYER_REGEN_ENABLED") -- register the events to be used
	Health:RegisterUnitEvent("PLAYER_REGEN_DISABLED") -- register the events to be used
	Health:RegisterUnitEvent("UPDATE_SHAPESHIFT_FORM")
	Health:RegisterUnitEvent("PLAYER_ROLES_ASSIGNED")
	Health:RegisterUnitEvent("GROUP_ROSTER_UPDATE")
	Health:RegisterUnitEvent("RAID_TARGET_UPDATE")

	Health:RegisterUnitEvent("UNIT_POWER_UPDATE", "player")
	Health:RegisterUnitEvent("UNIT_MAXHEALTH", "pet", "player")
	Health:RegisterUnitEvent("UNIT_MAXPOWER", "pet", "player")
	Health:RegisterUnitEvent("UNIT_AURA", "pet", "player");
	Health:RegisterUnitEvent("UNIT_COMBAT", "pet", "player")
	if not (DUF_GameVersion == "CLASSIC") then
		Health:RegisterUnitEvent("UNIT_EXITED_VEHICLE", "player");
	end
	Health:RegisterUnitEvent(DUF_UNIT_HEALTH_FREQUENT, "pet", "player") -- register the events to be used
	Health:RegisterUnitEvent("UNIT_POWER_FREQUENT", "pet", "player") -- Health bar is handling events for both bars
	-- PetEvents
	Health:RegisterUnitEvent("UNIT_PET", "player");
	Health:RegisterUnitEvent("PET_UI_UPDATE");
	Health:RegisterUnitEvent(DUF_UNIT_HAPPINESS, "pet")
	Health:RegisterUnitEvent("PET_ATTACK_START");
	Health:RegisterUnitEvent("PET_ATTACK_STOP");

	Health.PowerPoints = Health:CreateFontString("FontString","OVERLAY","GameTooltipText")
	Health.PowerPoints:SetFont("Fonts\\FRIZQT__.TTF", 27, "THINOUTLINE")
	Health.PowerPoints:SetTextColor(1, 0.9609375, 0.4140625);
	Health.PowerPoints:SetPoint("LEFT", Health, "RIGHT", 10, 0)		
	Health.PowerPoints:Show()

	Health:SetScript("OnEvent", function(self, event, ...) -- Health bar is handling events for both bars
		hasUI, isHunterPet = HasPetUI();
		local unit = ...
		if not ( (unit == "player") or (unit == "pet") ) then unit = "player" end
		-- Infrequent events
		if (
			event == "PLAYER_LOGIN" or
			event == "PLAYER_ENTERING_WORLD" or
			event == "PLAYER_REGEN_ENABLED" or
			event == "PLAYER_REGEN_DISABLED" or
			event == "PLAYER_ROLES_ASSIGNED" or
			event == "GROUP_ROSTER_UPDATE" or
			event == "UPDATE_SHAPESHIFT_FORM" or
			event == "UNIT_EXITED_VEHICLE" or
			event == "UNIT_PET" or
			event == "PET_UI_UPDATE" or
			event == "PET_ATTACK_START" or
			event == "PET_ATTACK_STOP") then
				UpdateHealth(Health, "player")
				UpdatePower(Power, "player")
				UpdateDruidMana(DruidMana, "player")
				UpdatePetUnit(hasUI, isHunterPet, "pet", event)
				if (DUF_GameVersion == "RETAIL") then
					DUFGetPowerpoints()
				end
				-- PlayerFrame:Hide() -- Hide PlayerFrame on login/events, it is obstinate ;)
		end

		-- More frequent events		
		if (
			event == "UNIT_AURA" or
			event == "UNIT_COMBAT" or
			event == "UNIT_POWER_UPDATE") then
				UpdateHealth(Health, "player")
				UpdatePower(Power, "player")
				UpdateDruidMana(DruidMana, "player")
		end
		-- Pet Happiness; note that Rogue/Druid combo points are on target health bars.
		if (
			event == DUF_UNIT_HAPPINESS) then
				UpdatePetUnit(hasUI, isHunterPet, "pet", event)
		end
		-- Most frequent health events		
		if (
			event == DUF_UNIT_HEALTH_FREQUENT or -- Fired when max health updates(MotW, Fort, Kings, bear form, etc.)
			event == "UNIT_MAXHEALTH" or
			event == "RAID_TARGET_UPDATE") then
				UpdateHealth(Health, "player")
				UpdatePetUnit(hasUI, isHunterPet, "pet", event)
		end
		-- Most frequent power events		
		if (
			event == "UNIT_POWER_FREQUENT" or
			event == "UNIT_MAXPOWER") then -- Fired when max power updates(MotW, AI, Kings, shapeshift, etc.)
				UpdatePower(Power, "player")
				UpdateDruidMana(DruidMana, "player")
				UpdatePetUnit(hasUI, isHunterPet, "pet", event)
				if (DUF_GameVersion == "RETAIL") then
					DUFGetPowerpoints() -- Even tho attached to Health bar it updates with the "UNIT_POWER_FREQUENT" event
				end
		end
	end)


	THealth:RegisterEvent("PLAYER_LOGIN") -- register the events to be used
	THealth:RegisterEvent("PLAYER_ENTERING_WORLD") -- register the events to be used
	THealth:RegisterEvent("PLAYER_ROLES_ASSIGNED") 
	THealth:RegisterEvent("GROUP_ROSTER_UPDATE") 
	THealth:RegisterEvent("PLAYER_TARGET_CHANGED") 
	THealth:RegisterEvent("RAID_TARGET_UPDATE") 
	THealth:RegisterUnitEvent(DUF_UNIT_HEALTH_FREQUENT, "target")
	THealth:RegisterUnitEvent("UNIT_POWER_FREQUENT", "target") 
	THealth:RegisterUnitEvent("UNIT_MAXHEALTH", "target")
	THealth:RegisterUnitEvent("UNIT_MAXPOWER", "target")
	THealth:RegisterUnitEvent("UNIT_POWER_UPDATE", "player", "COMBO_POINTS") -- This is the event that fires when you get a combo points; Blizzard apparently reused the pet happiness system for combo points. 

	THealth:SetScript("OnEvent", function(self, event, ...)
		if(UnitExists("target")) then
			if event == "UNIT_POWER_UPDATE" then
				DUFGetCombopoints(event)
			end
			if (event == "PLAYER_LOGIN") or
				(event == "PLAYER_ENTERING_WORLD") or
				event == "PLAYER_ROLES_ASSIGNED" or
				event == "GROUP_ROSTER_UPDATE" or
				(event == "PLAYER_TARGET_CHANGED") then
					finishFired = 0
					UpdateHealth(THealth, "target")
					UpdatePower(TPower, "target")
					DUFGetCombopoints(event)
			end
			if (event == DUF_UNIT_HEALTH_FREQUENT or -- initialise the health bar
			event == "UNIT_MAXHEALTH" or -- Fired when max health updates(MotW, Fort, Kings, bear form, etc.)
			event == "RAID_TARGET_UPDATE") then -- Fired each tick to update current health
					UpdateHealth(THealth, "target")
					DUFGetCombopoints(event)
			end
			if (event == "UNIT_POWER_FREQUENT" or
				event == "UNIT_MAXPOWER") then -- Fired when max power updates(MotW, AI, Kings, shapeshift, etc.)
					UpdatePower(TPower, "target")
			end
			THealth:SetAlpha(1)
			TPower:SetAlpha(1)
		else
			THealth:SetAlpha(0)
			TPower:SetAlpha(0)
		end
	end)

	ToTHealth:RegisterEvent("PLAYER_LOGIN") -- register the events to be used
	ToTHealth:RegisterEvent("PLAYER_ENTERING_WORLD") -- register the events to be used
	ToTHealth:RegisterEvent("PLAYER_TARGET_CHANGED") 
	ToTHealth:RegisterEvent("PLAYER_ROLES_ASSIGNED") 
	ToTHealth:RegisterEvent("GROUP_ROSTER_UPDATE") 
	ToTHealth:RegisterEvent("RAID_TARGET_UPDATE") 
	ToTHealth:RegisterUnitEvent(DUF_UNIT_HEALTH_FREQUENT, "targettarget")
	ToTHealth:RegisterUnitEvent("UNIT_POWER_FREQUENT", "targettarget") 
	ToTHealth:RegisterUnitEvent("UNIT_MAXHEALTH", "targettarget")
	ToTHealth:RegisterUnitEvent("UNIT_MAXPOWER", "targettarget")

	ToTHealth:SetScript("OnEvent", function(self, event, ...)
		----------
		if(UnitExists("targettarget")) then 
			if (event == "PLAYER_LOGIN" or -- initialise the power bars
				event == "PLAYER_ENTERING_WORLD" or
				event == "PLAYER_ROLES_ASSIGNED" or
				event == "GROUP_ROSTER_UPDATE" or
				event == "PLAYER_TARGET_CHANGED" or
				event == "RAID_TARGET_UPDATE") then
					UpdateHealth(ToTHealth, "targettarget")
			end
			if (event == DUF_UNIT_HEALTH_FREQUENT or -- initialise the health bar
			event == "UNIT_MAXHEALTH" or -- Fired when max health updates(MotW, Fort, Kings, bear form, etc.)
			event == "PLAYER_TARGET_CHANGED" or -- Fired when max health updates(MotW, Fort, Kings, bear form, etc.)
			event == "RAID_TARGET_UPDATE") then -- Fired each tick to update current health
					UpdateHealth(ToTHealth, "targettarget")
			end
			if (event == "PLAYER_TARGET_CHANGED" or -- initialise the power bars
				event == "UNIT_MAXPOWER" or -- Fired when max power updates(MotW, AI, Kings, shapeshift, etc.)
				event == "UNIT_POWER_FREQUENT") then -- Fired each tick to update current power
					UpdatePower(ToTPower, "targettarget")
			end
			ToTHealth:SetAlpha(1)
			ToTPower:SetAlpha(1)
		else
			ToTHealth:SetAlpha(0)
			ToTPower:SetAlpha(0)
		end
	end)

if not (DUF_GameVersion == "CLASSIC") then
	FocusHealth:RegisterEvent("PLAYER_LOGIN") -- register the events to be used
	FocusHealth:RegisterEvent("PLAYER_ENTERING_WORLD") -- register the events to be used
	FocusHealth:RegisterUnitEvent("PLAYER_FOCUS_CHANGED") -- register the events to be used
	FocusHealth:RegisterUnitEvent("PLAYER_ROLES_ASSIGNED") -- register the events to be used
	FocusHealth:RegisterUnitEvent("GROUP_ROSTER_UPDATE") -- register the events to be used
	FocusHealth:RegisterUnitEvent(DUF_UNIT_HEALTH_FREQUENT, "focus") -- register the events to be used
	FocusHealth:RegisterUnitEvent("UNIT_POWER_FREQUENT", "focus") -- FocusHealth bar is handling events for both bars
	FocusHealth:RegisterUnitEvent("UNIT_MAXHEALTH", "focus")
	FocusHealth:RegisterUnitEvent("UNIT_MAXPOWER", "focus")
	FocusHealth:RegisterUnitEvent("UNIT_COMBAT", "focus")
	FocusHealth:RegisterEvent("RAID_TARGET_UPDATE")

	FocusHealth:SetScript("OnEvent", function(self, event, ...) -- FocusHealth bar is handling events for both bars
		if(UnitExists("focus")) then
			if (event == "PLAYER_LOGIN") or
			(event == "PLAYER_ENTERING_WORLD") or
			(event == "PLAYER_FOCUS_CHANGED") or
			event == "PLAYER_ROLES_ASSIGNED" or
			event == "GROUP_ROSTER_UPDATE" or
			(event == "UNIT_COMBAT") then -- Fired each tick to update current health
				UpdateHealth(FocusHealth, "focus")
				UpdatePower(FocusPower, "focus")
			end
			if (event == DUF_UNIT_HEALTH_FREQUENT or -- Fired when max health updates(MotW, Fort, Kings, bear form, etc.)
			event == "UNIT_MAXHEALTH" or
			event == "RAID_TARGET_UPDATE") then
				UpdateHealth(FocusHealth, "focus")
			end
			if (event == "UNIT_POWER_FREQUENT" or
			event == "UNIT_MAXPOWER") then -- Fired when max power updates(MotW, AI, Kings, shapeshift, etc.)
				UpdatePower(FocusPower, "focus")
			end
			FocusHealth:Show()
			FocusPower:Show()
		else
			FocusHealth:Hide()
			FocusPower:Hide()
		end
	end)
end

-- FocusFrame:SetScript("OnShow", function(self) -- Hide FocusFrame after creation of DejaUFClassic FocusFrame
-- 	self:Hide()
-- end)

--Target Debuff Frames
local frameCount
local DUF_DebuffEvents = CreateFrame("Frame", DUF_DebuffEvents, UIParent)
	DUF_DebuffEvents:RegisterEvent("PLAYER_TARGET_CHANGED") 
	DUF_DebuffEvents:RegisterUnitEvent("UNIT_AURA", "target")

DUF_DebuffEvents:SetScript("OnEvent", function(self, event, ...)
	local size = 32
	local statusfontSize = size/3
	frameCount = 0
	for i=1,40 do
		local name, icon, count, _, duration, expirationTime, source = UnitDebuff("target",i)
		if (not UnitExists("target")) or ( source ~= "player") then
			if (_G["DUFDebuffFrame"..i] ~= nil) then
				_G["DUFDebuffFrame"..i]:Hide()
				_G["DeBuffCooldown"..i].stack:SetText("")
				_G["DUFDebuffFrame"..i].icon:SetTexture("")
				--Show Remaining Time Text
				_G["DeBuffCooldown"..i]:SetCooldown(0, 0)
				_G["DUFDebuffFrame"..i].fadeAnimation:Stop()
			end
			frameCount = 0
		elseif UnitExists("target") and ( source == "player") then
			frameCount = frameCount + 1
			if (_G["DUFDebuffFrame"..frameCount] == nil) then
				--Create Debuff Animation Frame
				local f = CreateFrame("Frame", "DUFDebuffFrame"..frameCount, UIParent)
					f:SetID(frameCount)
					f:SetSize(size, size)
					if frameCount == 1 then
						f:SetPoint("TOPLEFT", FizzleTHealth, "BOTTOMLEFT", 0, -20)		
					else
						f:SetPoint("TOPLEFT", _G["DUFDebuffFrame"..(frameCount-1)], "TOPRIGHT", 0, 0)		
					end

					--Create Debuff Icon Textures
					f.icon = f:CreateTexture(nil,"BACKGROUND")
					f.icon:SetAllPoints("DUFDebuffFrame"..frameCount)
					f.icon:SetMask("Interface\\ICONS\\Ability_Ambush")
					
					local DeBuffCooldown = CreateFrame("Cooldown", "DeBuffCooldown"..frameCount, f, "CooldownFrameTemplate")
					DeBuffCooldown:SetAllPoints()
					DeBuffCooldown:SetSwipeTexture("Interface\\ICONS\\Ability_Ambush")
					DeBuffCooldown:SetEdgeTexture("Interface\\AddOns\\DejaAuras\\Textures\\SetEdgeTexture_Clear");
					DeBuffCooldown:SetHideCountdownNumbers(false); -- Hides the countdown numbers regardless of the cvar setting

					-- Create Buff Stack Count Text Strings
					DeBuffCooldown.stack = DeBuffCooldown:CreateFontString("FontString","OVERLAY","GameTooltipText")
					DeBuffCooldown.stack:SetFont("Fonts\\FRIZQT__.TTF", statusfontSize, "THINOUTLINE")
					DeBuffCooldown.stack:SetTextColor(1, 1, 1);
					DeBuffCooldown.stack:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 0, 0)		
					DeBuffCooldown.stack:SetJustifyH("RIGHT")
					DeBuffCooldown.stack:SetJustifyV("BOTTOM")
			
					--Create Active Buff Fade Alpha In & Out Animations
					f.fadeAnimation = f:CreateAnimationGroup()
					f.fadeAnimation:SetLooping("REPEAT")

					local bufffadeout = f.fadeAnimation:CreateAnimation("Alpha")
					bufffadeout:SetFromAlpha(1);
					bufffadeout:SetToAlpha(0.33);
					bufffadeout:SetDuration(1);
					bufffadeout:SetOrder(1)

					local bufffadein = f.fadeAnimation:CreateAnimation("Alpha")
					bufffadein:SetFromAlpha(0.33);
					bufffadein:SetToAlpha(1);
					bufffadein:SetDuration(1);
					bufffadein:SetOrder(2)
			end
			_G["DUFDebuffFrame"..frameCount]:Show()
			_G["DUFDebuffFrame"..frameCount].icon:SetTexture(icon)
			if (count < 2) then
				_G["DeBuffCooldown"..frameCount].stack:SetText("")
			else
				_G["DeBuffCooldown"..frameCount].stack:SetText(count)
			end
			local timeremain = ((expirationTime - GetTime()) - duration)
			local StartTime = ( (GetTime() + timeremain) - 1 ) -- Minus one (-1) so it counts down to zero isntead of 1 and coincides with the default buff display.
			if ((GetTime() - StartTime) < duration) then 
				--Show Remaining Time Text
				_G["DeBuffCooldown"..frameCount]:SetCooldown( StartTime, duration)
				_G["DUFDebuffFrame"..frameCount].fadeAnimation:Play()
			end
		else
			_G["DUFDebuffFrame"..frameCount]:Hide()
		end
	end
end)

--Target Buff Frames
local DUF_BuffEvents = CreateFrame("Frame", DUF_BuffEvents, UIParent)
	DUF_BuffEvents:RegisterEvent("PLAYER_TARGET_CHANGED") 
	DUF_BuffEvents:RegisterUnitEvent("UNIT_AURA", "target")

DUF_BuffEvents:SetScript("OnEvent", function(self, event, ...)
	local size = 21
	local statusfontSize = size/3
	local frameCount = 0
	for i=1,BUFF_MAX_DISPLAY do
		local name, icon, count, _, duration, expirationTime, source, _, _, spellId  = UnitBuff("target",i)
		if _G["DUFBuffFrame"..i] then
			_G["DUFBuffFrame"..i]:Hide()
			_G["BuffCooldown"..i].stack:SetText("")
			_G["DUFBuffFrame"..i].icon:SetTexture("")
			--Clear Remaining Time Text
			-- _G["BuffCooldown"..i]:SetCooldown(0, 0)
			-- _G["DUFBuffFrame"..i].fadeAnimation:Stop()
		end
		if UnitExists("target") then
			frameCount = frameCount + 1
			if name then
				if (_G["DUFBuffFrame"..frameCount] == nil) then
					--Create Buff Animation Frame
					local f = CreateFrame("Frame", "DUFBuffFrame"..frameCount, UIParent, "TargetBuffFrameTemplate")
						f:SetID(frameCount);
						f:SetSize(size, size)
						f:Hide()
						if frameCount == 1 then
							f:SetPoint("BOTTOMLEFT", FizzleTHealth, "TOPLEFT", 0, 30)	
						else
							f:SetPoint("BOTTOMLEFT", _G["DUFBuffFrame"..(frameCount-1)], "BOTTOMRIGHT", 4, 0)		
						end
						--Create Buff Icon Textures
						f.icon = f:CreateTexture(nil,"BACKGROUND")
						f.icon:SetAllPoints("DUFBuffFrame"..frameCount)
						f.icon:SetMask("Interface\\ICONS\\Ability_Ambush")
						
						local BuffCooldown = CreateFrame("Cooldown", "BuffCooldown"..frameCount, f, "CooldownFrameTemplate")
						BuffCooldown:SetAllPoints()
						BuffCooldown:SetSwipeTexture("Interface\\ICONS\\Ability_Ambush")
						BuffCooldown:SetEdgeTexture("Interface\\AddOns\\DejaAuras\\Textures\\SetEdgeTexture_Clear");
						BuffCooldown:SetHideCountdownNumbers(false); -- Hides the countdown numbers regardless of the cvar setting

						-- Create Buff Stack Count Text Strings
						BuffCooldown.stack = BuffCooldown:CreateFontString("FontString","OVERLAY","GameTooltipText")
						BuffCooldown.stack:SetFont("Fonts\\FRIZQT__.TTF", statusfontSize, "THINOUTLINE")
						BuffCooldown.stack:SetTextColor(1, 1, 1);
						BuffCooldown.stack:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 0, 0)		
						BuffCooldown.stack:SetJustifyH("RIGHT")
						BuffCooldown.stack:SetJustifyV("BOTTOM")

					local tooltip = CreateFrame("GameTooltip","DUFBuffDisplayTooltip"..frameCount,UIParent,"GameTooltipTemplate")
					tooltip:SetPoint("BOTTOMLEFT", f, "TOPLEFT")

					f:SetScript('OnEnter', function()
						local index = f:GetID();
						-- print(index)
						tooltip:ClearLines()
						tooltip:SetOwner(UIParent,"ANCHOR_PRESERVE")
						tooltip:SetUnitAura("target",index)
						tooltip:Show()				
					end)
					
					f:SetScript('OnLeave', function()
						local index = f:GetID();
						-- print(index)
						_G["DUFBuffDisplayTooltip"..index]:Hide()
					end)
					local index = f:GetID();
					-- print(index)
					_G["DUFBuffFrame"..index]:Hide()

					--Clear Remaining Time Text
					_G["BuffCooldown"..i].stack:SetText("")
					_G["BuffCooldown"..i]:SetCooldown(0, 0)
				end
				if i == 1 then
					_G["DUFBuffFrame1"]:ClearAllPoints()
					_G["DUFBuffFrame1"]:SetPoint("BOTTOMLEFT", FizzleTHealth, "TOPLEFT", 0, 30)	
				end
				if i == 9 then
					_G["DUFBuffFrame9"]:ClearAllPoints()
					_G["DUFBuffFrame9"]:SetPoint("BOTTOMLEFT", FizzleTHealth, "TOPLEFT", 0, 30)	
					_G["DUFBuffFrame1"]:ClearAllPoints()
					_G["DUFBuffFrame1"]:SetPoint("BOTTOMLEFT", _G["DUFBuffFrame9"], "TOPLEFT", 0, 0)		
				end
				_G["DUFBuffFrame"..frameCount]:Show()
				_G["DUFBuffFrame"..frameCount].icon:SetTexture(icon)
				if count and (count < 2) then
					_G["BuffCooldown"..frameCount].stack:SetText("")
				else
					_G["BuffCooldown"..frameCount].stack:SetText(count)
				end
				if expirationTime then
					local timeremain = ((expirationTime - GetTime()) - duration)
					local StartTime = ( (GetTime() + timeremain) - 1 ) -- Minus one (-1) so it counts down to zero isntead of 1 and coincides with the default buff display.
					if ((GetTime() - StartTime) < duration) then 
						--Show Remaining Time Text
						_G["BuffCooldown"..frameCount]:SetCooldown( StartTime, duration)
						-- _G["DUFBuffFrame"..frameCount].fadeAnimation:Play()
						-- CreateBuff(name, icon, count, duration, expirationTime, frameCount)
					end
				end
			end
		end
	end
end)